import { Parser } from './parser';
import { getSiteNames } from './utility';

export interface SiteHttps {
  ports: number[];
  cert: string;
  redirect?: boolean;
  hsts?: boolean;
}

export interface Header {
  name: string;
  value: string;
}

export interface SiteRenderData {
  domains: string[];
  ports: number[];
  https?: SiteHttps;
  headers?: Header[];
  normalizeDomain?: string;
  minio?: boolean;
  disableTop?: boolean;
  noVerifyCerts?: boolean;
  noBuffer?: boolean;
  noCache?: boolean;
  upstream: string;
  timeout?: number;
  serverExtra?: string[];
  locationExtra?: string[];
}

export interface RenderData {
  purgeAllowed?: string[];
  externalRealIp?: boolean;
  realIp?: string[];
  limitRate?: string;
  limitBurst?: string;
  maxCacheSize: string;
  dhparamPath: string;
  ticketKeyPath: string;
  certsPath: string;
  sites: SiteRenderData[];
  httpExtra?: string[];
}

function getSiteData(
  domain: string,
  input: Record<string, string> = process.env,
): SiteRenderData {
  const parser = new Parser(`SITE_${domain}_`, input);
  let https: SiteHttps;
  const httpsCert = parser.getString('HTTPS');
  if (httpsCert) {
    https = {
      cert: httpsCert,
      ports: parser.getArrayNumber('HTTPS_PORT') || [443],
      redirect: !parser.getBoolean('HTTPS_NOREDIR'),
      hsts: parser.getBoolean('HSTS'),
    };
  }
  return {
    domains: domain.split('+'),
    ports: parser.getArrayNumber('PORT') || [80],
    https,
    headers: Object.entries(parser.getDict('HEADER')).map(([name, value]) => ({
      name: name.replace(/_/g, '-'),
      value,
    })),
    normalizeDomain: parser.getString('NORMALIZE_DOMAIN'),
    minio: parser.getBoolean('MINIO'),
    disableTop: parser.getBoolean('DISABLE_TOP'),
    noVerifyCerts: parser.getBoolean('NO_VERIFY_CERTS'),
    noBuffer: parser.getBoolean('NO_BUFFER'),
    noCache: parser.getBoolean('NO_CACHE'),
    upstream: input[`SITE_${domain}`],
    timeout: parser.getNumber('TIMEOUT'),
    serverExtra: parser.getArray('SERVER_EXTRA'),
    locationExtra: parser.getArray('LOCATION_EXTRA'),
  };
}

export function getData(
  input: Record<string, string> = process.env,
): RenderData {
  const parser = new Parser('', input);
  return {
    purgeAllowed: parser.getArray('PURGE_ALLOWED'),
    externalRealIp: parser.getBoolean('EXTERNAL_REAL_IP'),
    realIp: parser.getArray('REAL_IP'),
    limitRate: parser.getString('LIMIT_RATE'),
    limitBurst: parser.getString('LIMIT_BURST'),
    maxCacheSize: parser.getString('MAX_CACHE_SIZE') || '10g',
    dhparamPath:
      parser.getString('DHPARAM_PATH') || '/etc/nginx/generated/dhparam.pem',
    ticketKeyPath:
      parser.getString('TICKET_KEY_PATH') || '/etc/nginx/generated/ticket.key',
    certsPath: parser.getString('CERTS_PATH') || '/etc/nginx/certs',
    sites: getSiteNames().map((domain) => getSiteData(domain, input)),
    httpExtra: parser.getArray('HTTP_EXTRA'),
  };
}
